package com.pj.api;


import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.StrUtil;
import com.pj.current.satoken.StpUserUtil;
import com.pj.project.ocr.OcrService;
import com.pj.project.ocr.bo.OcrResult;
import com.pj.project.re_mini_user_car_filing.ReMiniUserCarFiling;
import com.pj.project.re_mini_user_car_filing.ReMiniUserCarFilingService;
import com.pj.project.re_mini_user_person_filing.ReMiniUserPersonFiling;
import com.pj.project.re_mini_user_person_filing.ReMiniUserPersonFilingService;
import com.pj.project.sp_admin.SpAdminService;
import com.pj.project.tb_car_filing.TbCarFiling;
import com.pj.project.tb_car_filing.TbCarFilingService;
import com.pj.project.tb_dept.TbDeptService;
import com.pj.project.tb_mini_user.TbMiniUser;
import com.pj.project.tb_mini_user.TbMiniUserService;
import com.pj.project.tb_person_filing.TbPersonFiling;
import com.pj.project.tb_person_filing.TbPersonFilingService;
import com.pj.project.tb_person_visit_record.TbPersonVisitRecord;
import com.pj.project.tb_person_visit_record.TbPersonVisitRecordService;
import com.pj.project.wx.WxService;
import com.pj.utils.sg.AjaxJson;
import com.pj.utils.so.SoMap;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.weaver.loadtime.Aj;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

@RequestMapping("wx")
@RestController
@Slf4j
public class WxController {
    @Resource
    private WxService wxService;
    @Resource
    private TbMiniUserService tbMiniUserService;
    @Resource
    private TbPersonFilingService tbPersonFilingService;
    @Resource
    private ReMiniUserPersonFilingService reMiniUserPersonFilingService;
    @Resource
    private TbCarFilingService tbCarFilingService;
    @Resource
    private ReMiniUserCarFilingService reMiniUserCarFilingService;
    @Resource
    private TbPersonVisitRecordService tbPersonVisitRecordService;

    @Resource
    private SpAdminService spAdminService;

    @Resource
    private OcrService ocrService;
    @Resource
    private TbDeptService tbDeptService;


    @GetMapping(value = "getRedirectUrl")
    public AjaxJson getRedirectUrl(String path, @RequestParam(required = false) String state) {
        return AjaxJson.getSuccessData(wxService.getRedirectUrl(path, state));
    }

    @GetMapping(value = "getOpenidByCode")
    public AjaxJson getOpenidByCode(String code, @RequestParam(required = false) String openid) {
        return AjaxJson.getSuccessData(wxService.getOpenidByCode(code, openid));
    }


    @GetMapping(value = "getWxConfig")
    public AjaxJson getWxConfig(String url) {
        return AjaxJson.getSuccessData(wxService.getWxConfig(url));
    }

    @GetMapping(value = "person-filing")
    public AjaxJson getPersonFiling() {
        SoMap so = SoMap.getRequestSoMap();
        so.put("miniUserId", StpUserUtil.getMiniUserId());
        if (StpUserUtil.isInnerMiniUser()) {
            //内部人员 自己备案的+部门的备案
           if (!StpUserUtil.isAdmin()){
               so.put("departmentId", StpUserUtil.getDepartmentId());
           }
            List<TbPersonFiling> list = tbPersonFilingService.getForWx(so.startPage());
            return AjaxJson.getPageData(so.getDataCount(), list);
        } else {
            //备案自己的+帮别人备案的
            List<TbPersonFiling> list = tbPersonFilingService.getOuterList(so.startPage());
            return AjaxJson.getPageData(so.getDataCount(), list);
        }
    }

    @PostMapping("addPersonFiling")
    public AjaxJson addPersonFiling(TbPersonFiling tbPersonFiling) {
        if (!StpUserUtil.isInnerMiniUser()) {
            tbPersonFiling.setDeptId(StpUserUtil.getDeptId());
        }
        TbPersonFiling db = tbPersonFilingService.findByIdCard(tbPersonFiling.getIdCard());
        if (db != null) {
            tbPersonFiling = db;
        } else {
            tbPersonFilingService.add(tbPersonFiling);
        }
        ReMiniUserPersonFiling reMiniUserPersonFiling = new ReMiniUserPersonFiling();
        reMiniUserPersonFiling.setFilingPersonId(tbPersonFiling.getId())
                .setMiniUserId(StpUserUtil.getMiniUserId());
        reMiniUserPersonFilingService.save(reMiniUserPersonFiling);
        return AjaxJson.getSuccess();
    }

    @PostMapping("editPersonFiling")
    public AjaxJson editPersonFiling(TbPersonFiling tbPersonFiling) {
        tbPersonFilingService.update(tbPersonFiling);
        return AjaxJson.getSuccess();
    }

    @PostMapping("deletePersonFiling")
    public AjaxJson deletePersonFiling(Integer id) {
        tbPersonFilingService.delete(id);
        return AjaxJson.getSuccess();
    }

    @PostMapping("addCarFiling")
    public AjaxJson addCarFiling(TbCarFiling tbCarFiling) {
        if (!StpUserUtil.isInnerMiniUser()) {
            tbCarFiling.setDeptId(StpUserUtil.getDeptId());
        }
        tbCarFilingService.add(tbCarFiling);
        ReMiniUserCarFiling reMiniUserCarFiling = new ReMiniUserCarFiling();
        reMiniUserCarFiling.setFilingCarId(tbCarFiling.getId())
                .setMiniUserId(StpUserUtil.getMiniUserId());
        reMiniUserCarFilingService.save(reMiniUserCarFiling);
        return AjaxJson.getSuccess();
    }

    @GetMapping("car-filing")
    public AjaxJson getCarFiling() {
        SoMap so = SoMap.getRequestSoMap();
        so.put("miniUserId", StpUserUtil.getMiniUserId());
        if (StpUserUtil.isAdmin()) {
            //内部人员 自己备案的+部门的备案
            if (!StpUserUtil.isAdmin()) {
                so.put("departmentId", StpUserUtil.getDepartmentId());
            }
            List<TbCarFiling> list = tbCarFilingService.getForWx(so.startPage());
            return AjaxJson.getPageData(so.getDataCount(), list);
        } else {
            //备案自己的+帮别人备案的
            List<TbCarFiling> list = tbCarFilingService.getOuterList(so.startPage());
            return AjaxJson.getPageData(so.getDataCount(), list);
        }


    }

    @PostMapping("deleteCarFiling")
    public AjaxJson deleteCarFiling(Long id) {
        tbCarFilingService.delete(id);
        return AjaxJson.getSuccess();
    }

    @PostMapping("loginByOpenid")
    public AjaxJson loginByOpenid() {
        SoMap soMap = SoMap.getRequestSoMap();
        String openid = soMap.getString("openid");
        Integer type = soMap.getInt("type");
        if (StrUtil.isEmpty(openid)) {
            return AjaxJson.getError("参数异常");
        }
        return tbMiniUserService.loginByOpenid(openid, type);
    }

    @RequestMapping("solve")
    public AjaxJson solve() {
        SoMap soMap = SoMap.getRequestSoMap();
        tbPersonVisitRecordService.solve(soMap);
        return AjaxJson.getSuccess();
    }

    /**
     * 绑定
     *
     * @param openid
     * @param key
     * @param password
     * @param type
     * @param name
     * @return
     */
    @PostMapping("bind")
    public AjaxJson bind(String openid, String key, String password, Integer type, String name) {
        return spAdminService.bindUser(openid, key, password, type, name);
    }

    /**
     * 登录
     *
     * @param openid
     * @param key
     * @param password
     * @return
     */
    @PostMapping("login")
    public AjaxJson login(String openid, String key, String password) {
        return tbMiniUserService.login(openid, key, password);
    }

    /**
     * 识别边民证，弃用
     */
    @RequestMapping("ocrIcCard")
    public AjaxJson ocrIcCard(String icCardUrl) {
        OcrResult ocrResult = ocrService.ocrIcCard(icCardUrl);
        return AjaxJson.getSuccessData(ocrResult);
    }

    /**
     * 识别身份证，弃用
     */
    @RequestMapping("ocrIdCard")
    @Deprecated
    public AjaxJson ocrIdCard(String idCardUrl) {
        OcrResult ocrResult = ocrService.ocrIdCard(idCardUrl);
        return AjaxJson.getSuccessData(ocrResult);
    }

    /**
     * 识别身份证或边民证
     */
    @RequestMapping("ocrIdOrIcCard")
    public AjaxJson ocrIdOrIcCard(String cardUrl) {
        OcrResult ocrResult = ocrService.ocrIdOrIc(cardUrl);
        return AjaxJson.getSuccessData(ocrResult);
    }

}
