package com.pj.current.mybatis;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.log.StaticLog;
import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import com.pj.current.satoken.StpUserUtil;
import org.apache.ibatis.reflection.MetaObject;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.util.Date;
import java.util.Objects;
import java.util.function.Supplier;

/**
 *
 */
@Component
public class MyBatisPlusMetaObjectHandler implements MetaObjectHandler {

    private static final String CREATE_TIME = "createTime";
    private static final String CREATED_BY = "createBy";
    private static final String UPDATE_TIME = "updateTime";
    private static final String UPDATE_BY = "updateBy";


    /**
     * 插入填充
     *
     * @param metaObject 元数据对象
     */
    @Override
    public void insertFill(MetaObject metaObject) {
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (servletRequestAttributes==null){
            return;
        }
        String createBy = "";
        if (StpUtil.isLogin()) {
            createBy = StpUserUtil.getAdminName();
        }
        StaticLog.info("create by:{}", createBy);
        if (metaObject.hasGetter(CREATED_BY)) {
            strictInsertFill(metaObject, CREATED_BY, String.class, createBy);
        }
        if (metaObject.hasGetter(CREATE_TIME)) {
            strictInsertFill(metaObject, CREATE_TIME, Date.class, new Date());
        }
        if (metaObject.hasGetter(UPDATE_BY)) {
            strictUpdateFill(metaObject, UPDATE_BY, String.class, createBy);
        }
        if (metaObject.hasGetter(UPDATE_TIME)) {
            strictUpdateFill(metaObject, UPDATE_TIME, Date.class, new Date());
        }
    }

    /**
     * 更新填充
     *
     * @param metaObject 元数据对象
     */
    @Override
    public void updateFill(MetaObject metaObject) {
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (servletRequestAttributes==null){
            return;
        }
        if (metaObject.hasGetter(UPDATE_BY)) {
            strictUpdateFill(metaObject, UPDATE_BY, String.class, StpUserUtil.getAdminName());
        }
        if (metaObject.hasGetter(UPDATE_TIME)) {
            strictUpdateFill(metaObject, UPDATE_TIME, Date.class, new Date());
        }
    }

    @Override
    public MetaObjectHandler strictFillStrategy(MetaObject metaObject, String fieldName, Supplier<?> fieldVal) {
        Object value = metaObject.getValue(fieldName);
        if (value == null
                || StrUtil.isEmpty(value.toString())
                || StrUtil.equals(fieldName, UPDATE_BY)
                || StrUtil.equals(fieldName, UPDATE_TIME)) {
            Object obj = fieldVal.get();
            if (Objects.nonNull(obj)) {
                metaObject.setValue(fieldName, obj);
            }
        }
        return this;
    }
}
