package com.pj.project.tb_department;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.utils.so.*;
import com.pj.project4sp.SP;

import com.pj.current.satoken.StpUserUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;


/**
 * Controller: tb_customer -- 客户管理
 *
 * @author qzyReal
 */
@RestController
@RequestMapping("/TbDepartment/")
public class TbDepartmentController {

    /**
     * 底层 Service 对象
     */
    @Autowired
    TbDepartmentService tbDepartmentService;



    /**
     * 增
     */
    @RequestMapping("add")
    @SaCheckPermission(TbDepartment.PERMISSION_CODE_ADD)
    public AjaxJson add(TbDepartment t) {
        t.setStatus(0);
        tbDepartmentService.add(t);
        return AjaxJson.getSuccessData(t);
    }

    /**
     * 删
     */
    @RequestMapping("delete")
    @SaCheckPermission(TbDepartment.PERMISSION_CODE_DEL)
    public AjaxJson delete(String id) {
        tbDepartmentService.delete(id);
        return AjaxJson.getSuccess();
    }


    /**
     * 改
     */
    @RequestMapping("update")
    @SaCheckPermission(TbDepartment.PERMISSION_CODE_EDIT)
    public AjaxJson update(TbDepartment t) {
        tbDepartmentService.updateById(t);
        return AjaxJson.getSuccess();
    }

    /**
     * 查 - 根据id
     */
    @RequestMapping("getById")
    public AjaxJson getById(String id) {
        TbDepartment t = tbDepartmentService.getById(id);
        return AjaxJson.getSuccessData(t);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    @RequestMapping("getList")
    public AjaxJson getList() {
        SoMap so = SoMap.getRequestSoMap();
        if (!StpUserUtil.isAdmin()){
            so.put("id",StpUserUtil.getDeptId());
        }
        List<TbDepartment> list = tbDepartmentService.getList(so.startPage());
        return AjaxJson.getPageData(so.getDataCount(), list);
    }


    /**
     * 改 - 状态(0=否,1=是)
     */
    @RequestMapping("updateStatus")
    @SaCheckPermission(TbDepartment.PERMISSION_CODE)
    public AjaxJson updateStatus(String id, String value) {
        int line = SP.publicMapper.updateColumnById(TbDepartment.TABLE_NAME, "status", value, id);
        return AjaxJson.getByLine(line);
    }


}
