package com.pj.project.tb_person_filing;

import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.PhoneUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;

import com.pj.project.tb_dept.TbDept;
import com.pj.project.tb_dept.TbDeptService;
import com.pj.project.tb_venues.TbVenues;
import com.pj.project.tb_venues.TbVenuesService;
import com.pj.utils.sg.AjaxError;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

@Component
public class ImportPersonListener extends AnalysisEventListener<ImportPersonBO> {
    @Resource
    @Lazy
    private TbPersonFilingService tbPersonFilingService;

    @Resource
    private TbDeptService tbDeptService;
    @Resource
    private TbVenuesService tbVenuesService;


    public List<ImportPersonBO> list = new ArrayList<>();

    @Override
    public void invoke(ImportPersonBO importPersonBO, AnalysisContext analysisContext) {
        if (StrUtil.isNotEmpty(importPersonBO.getDeptName())) {
            list.add(importPersonBO);
        }
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        int index = 1;

        list.stream()
                .collect(Collectors.groupingBy(ImportPersonBO::getDeptName))
                .forEach((deptName, deptList) -> {
                    TbDept tbDept = tbDeptService.findByName(deptName);
                    if (tbDept == null) {
                        throw new AjaxError("组织不存在");
                    }
                    //某场所所有的备案
                    boolean autoJudge = TbDept.NeedJudgeEnum.AUTO_JUDGE.getType().equals(tbDept.getNeedJudge());
                    deptList.stream().collect(Collectors.groupingBy(ImportPersonBO::getVenuesName))
                            .forEach((venuesName, venuesList) -> {
                                TbVenues tbVenues = tbVenuesService.findByName(venuesName);
                                if (tbVenues == null) {
                                    throw new AjaxError("备案场所不存在");
                                }
                                List<TbPersonFiling> personFilings = new ArrayList<>();
                                venuesList.forEach(importPersonBO -> {
                                    String name = importPersonBO.getName();
                                    if (StrUtil.isEmpty(name)) {
                                        throw new AjaxError("表格中存在姓名为空的行");
                                    }
                                    String phone = importPersonBO.getPhone();
                                    if (!PhoneUtil.isPhone(phone)) {
                                        throw new AjaxError(phone + "手机号码不正确");
                                    }
                                    String idCard = importPersonBO.getIdCard();
                                    if (!IdcardUtil.isValidCard(idCard)) {
                                        throw new AjaxError(idCard + "身份证号不正确");
                                    }
                                    TbPersonFiling tbPersonFiling = tbPersonFilingService.findByIdCard(idCard);
                                    if (tbPersonFiling == null) {
                                        tbPersonFiling = new TbPersonFiling();
                                    }
                                    tbPersonFiling.setType(importPersonBO.getType().equals("内部") ? 2 : 1).setName(name)
                                            .setPhone(phone).setPin(idCard).setIdCard(idCard).setDeptId(tbDept.getId())
                                            .setDeptName(tbDept.getName()).setDepartmentId(tbVenues.getDepartmentId()).setVenuesId(tbVenues.getId());
                                    if (autoJudge) {
                                        tbPersonFiling.setJudgeState(1)
                                                .setJudgeTime(new Date())
                                                .setJudgeBy("平台自动审核");
                                    }
                                    personFilings.add(tbPersonFiling);
                                });
                                tbPersonFilingService.saveOrUpdateBatch(personFilings);
                                //自动下发到该场所以及等级低于等于的场所
                                if (autoJudge) {
                                        tbPersonFilingService.issued(tbVenues,personFilings);
                                }
                            });
                });
    }
}
