package com.pj.project.tb_person_filing;

import java.io.Serializable;
import java.util.Date;

import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.EqualsAndHashCode;

import lombok.Data;
import lombok.experimental.Accessors;
import org.springframework.format.annotation.DateTimeFormat;

/**
 * Model: tb_person_filing -- 人员备案
 *
 * @author qzyReal
 */
@Data
@Accessors(chain = true)
@TableName(TbPersonFiling.TABLE_NAME)
@EqualsAndHashCode(callSuper = false)
public class TbPersonFiling extends Model<TbPersonFiling> implements Serializable {

    // ---------- 模块常量 ----------
    /**
     * 序列化版本id
     */
    private static final long serialVersionUID = 1L;
    /**
     * 此模块对应的表名
     */
    public static final String TABLE_NAME = "tb_person_filing";
    /**
     * 此模块对应的权限码
     */
    public static final String PERMISSION_CODE_LIST = "tb-person-filing-list";
    public static final String PERMISSION_CODE_ADD = "tb-person-filing-add";
    public static final String PERMISSION_CODE_EDIT = "tb-person-filing-edit";
    public static final String PERMISSION_CODE_DEL = "tb-person-filing-del";
    public static final String PERMISSION_CODE_JUDGE = "tb-person-filing-judge";
    public static final String PERMISSION_CODE_IMPORT = "tb-person-filing-import";


    // ---------- 表中字段 ----------
    /**
     *
     */
    @TableId(type = IdType.AUTO)
    private Integer id;
    /**
     * 场所ID
     */
    private Long venuesId;
    /**
     * 部门ID
     */
    private String departmentId;
    /**
     * 组织ID
     */
    private Long deptId;

    /**
     * 姓名
     */
    private String name;
    /**
     * 工号
     */
    private String pin;
    private String phone;


    /**
     * 类型(1=临时,2=内部)
     */
    private Integer type = 1;

    /**
     * 身份证
     */
    private String idCard;
    /**
     * 身份证图片url
     */
    private String idCardUrl;

    /**
     * 来访事由
     */
    private String visitReason;

    /**
     * 预计来访时间
     */
    private Date preComeTime;

    /**
     * 预计结束时间
     */
    private Date preLeaveTime;

    /**
     * 状态
     */
    private String visit;

    private String photo;

    /**
     * 备注
     */
    private String remark;

    /**
     * 创建时间
     */
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    /**
     * 创建人
     */
    @TableField(fill = FieldFill.INSERT)
    private String createBy;

    /**
     * 更新时间
     */
    @TableField(fill = FieldFill.UPDATE)
    private Date updateTime;

    /**
     * 更新人
     */
    @TableField(fill = FieldFill.UPDATE)
    private String updateBy;

    private int judgeState;
    private Date judgeTime;
    private String judgeBy;
    private String openid;
    /**
     * 边民证号
     */
    private String icCard;
    /**
     * 边民证图片url
     */
    private String icCardUrl;
    /**
     * 边民证或身份证图片url
     */
    private String cardUrl;
    /**
     * 边民证有效期，起始日期
     */

    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date icCardStartTime;
    /**
     * 边民证有效期，到期时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date icCardEndTime;

    @TableField(exist = false)
    private String deptName;

    @TableField(exist = false)
    private String idCadStr;
    @TableField(exist = false)
    private String venuesName;

    @TableField(exist = false)
    private String departmentName;

    public String getIdCadStr() {
        String str = IdcardUtil.getBirthByIdCard(idCard);
        if (StrUtil.isNotEmpty(str)) {
            return idCard.replace(str, "********");
        }
        return idCard;
    }
}
