package com.pj.project.tb_venues;

import java.util.Date;
import java.util.List;

import cn.dev33.satoken.stp.StpUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pj.current.global.BusinessException;
import com.pj.project.tb_dept.TbDept;
import com.pj.project.tb_dept.TbDeptService;
import com.pj.project.tb_gate_terminal.TbGateTerminal;
import com.pj.project.tb_gate_terminal.TbGateTerminalService;
import com.pj.project.tb_terminal.TbTerminal;
import com.pj.project.tb_terminal.TbTerminalService;
import com.pj.utils.sg.AjaxError;
import com.pj.utils.sg.AjaxJson;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.pj.utils.so.*;

import javax.annotation.Resource;

/**
 * Service: tb_venues -- 场所管理
 *
 * @author qzyReal
 */
@Service
@Transactional(rollbackFor = Exception.class)
@Slf4j
public class TbVenuesService extends ServiceImpl<TbVenuesMapper, TbVenues> implements IService<TbVenues> {

    /**
     * 底层 Mapper 对象
     */
    @Autowired
    TbVenuesMapper tbVenuesMapper;
    @Resource
    private TbDeptService tbDeptService;

    @Resource
    private TbTerminalService tbTerminalService;

    @Resource
    private TbGateTerminalService tbGateTerminalService;

    /**
     * 增
     */
    public void add(TbVenues t) {
        if (t.getPId() == -1) {
            String name = t.getName();
            TbVenues db = findByName(name);
            if (db != null) {
                throw new AjaxError("场所已存在");
            }
        }
        this.save(t);
    }


    /**
     * 删
     */
    public void delete(Long id) {
        List<TbVenues> list = this.findByPid(id);
        if (!list.isEmpty()) {
            throw new BusinessException("该场所存在通道，不能删除");
        }
        this.removeById(id);
    }

    private List<TbVenues> findByPid(Long id) {
        QueryWrapper<TbVenues> ew = new QueryWrapper<>();
        ew.eq("p_id", id);
        return list(ew);
    }

    /**
     * 改
     */
    public void update(TbVenues t) {
        if (t.getPId() == -1) {
            String name = t.getName();
            TbVenues db = findByName(name);
            if (db != null&&t.getId().longValue()!=db.getId().longValue()) {
                throw new AjaxError("场所名已被占用");
            }
        }
        this.updateById(t);
    }

    /**
     * 查
     */
    public TbVenues getById(Long id) {
        return super.getById(id);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    public List<TbVenues> getList(SoMap so) {
        return tbVenuesMapper.getList(so);
    }


    public List<TbVenues> getInnerList(SoMap so) {
        return tbVenuesMapper.getInnerList(so);
    }

    /**
     * 开门
     * @param id
     * @return
     */
    public AjaxJson release(Long id) {
        //人脸设备
        List<TbTerminal> tbTerminalList = tbTerminalService.findByChannelId(id);
        tbTerminalList.forEach(tbTerminal -> tbTerminalService.open(tbTerminal.getId()));
        List<TbGateTerminal> tbGateTerminals = tbGateTerminalService.findByChannelId(id);
        boolean flag = tbTerminalList.isEmpty() && tbGateTerminals.isEmpty();
        if (flag) {
            return AjaxJson.getError("该通道无设备");
        }
        tbGateTerminals.stream().filter(tbGateTerminal -> tbGateTerminal.getDirection() == 1)
                .forEach(tbGateTerminal -> {
                    try {
                        tbGateTerminalService.open(tbGateTerminal.getId());
                    } catch (Exception e) {
                        log.error("开道闸失败:{}", e.getMessage());
                        throw new BusinessException("道闸通讯异常");
                    }
                });
        return AjaxJson.getSuccess();
    }

    public void setting(long id, int status) {
        TbVenues tbVenues = this.getById(id);
        tbVenues.setStatus(status).setUpdateTime(new Date());
        List<TbVenues> list = findByPid(id);
        list.forEach(v -> v.setStatus(status).setUpdateTime(new Date()));
        if (tbVenues.getPId() != -1 && status == 1) {
            TbVenues parent = this.getById(tbVenues.getPId());
            parent.setStatus(status);
            list.add(parent);
        }
        list.add(tbVenues);
        this.updateBatchById(list);
    }

    /**
     * 根据名称查询
     *
     * @param venuesName
     * @return
     */
    public TbVenues findByName(String venuesName) {
        QueryWrapper<TbVenues> ew = new QueryWrapper<>();
        ew.lambda().eq(TbVenues::getName, venuesName)
                .eq(TbVenues::getPId, -1);
        return getOne(ew);
    }

    /**
     * 查找小于等于level等级的场所
     * @param level
     * @return
     */
    public List<TbVenues> getLowByLevel(Integer level) {
        QueryWrapper<TbVenues> ew = new QueryWrapper<>();
        ew.lambda().ge(TbVenues::getLevel,level).eq(TbVenues::getPId,-1);
        return list(ew);
    }

    /**
     * 通道
     * @param pId
     * @return
     */
    public List<TbVenues> getByPid(Long pId) {
        QueryWrapper<TbVenues> ew = new QueryWrapper<>();
        ew.lambda().eq(TbVenues::getPId,pId);
        return list(ew);
    }

    /**
     * 查找比之等级高的场所
     * @param level
     * @return
     */
    public List<TbVenues> getHightLevel(Integer level) {
        QueryWrapper<TbVenues> ew = new QueryWrapper<>();
        ew.lambda().le(TbVenues::getLevel,level);
        return list(ew);
    }
}
